package PutFiles ;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(put_files);

use Reporting qw(start_report failed_report ok_report debug_report report_and_die);
use Dump2File        qw(dump2file);

use strict;

#
# Write files sent by the NJS
#
# Called in response to a TSI_PUTFILES request from the NJS
#
# arg1: none
#
# returns: void to TSI
#
#          takes over NJS-TSI protocol to receive files.
#
#
# (writes to the NJS COMMAND and DATA sockets)
#

sub put_files {

    my $output;
    my $file_name;
    my $packet_length;
    my $left;
    my $offset;
    my $len;
    my $bytes;
    my $written;
    my $action;
    my $from_njs = shift;

    start_report("Receiving files.");

    # Extract the word following the marker
    ($action) = ($from_njs =~ /.*#TSI_FILESACTION (.+)\n/);
	$action = 1 unless length $action; # default to new only for old NJSs

     # OK, ready to go into mode to accept files and data
    print main::CMD_SOCK "TSI_OK\n";
    print main::CMD_SOCK "ENDOFMESSAGE\n";

    # Read the file name and mode
    $file_name = <main::CMD_SOCK>;
    chomp $file_name;         # gets rid of ENFOFMESSAGE
    $file_name =~ s/\n.*//g;  # keep only first line and without new line char
    my $mode;
 
    $mode=substr($file_name, length($file_name)-1, 1); #just want last char
    $file_name=substr($file_name, 0, length($file_name)-2);
    
    while($file_name ne "-1") {

		dump2file($file_name);

		$file_name =~ s/\$USER/$ENV{USER}/g;
		$file_name =~ s/\$HOME/$ENV{HOME}/g;

		# Untaint (I trust the source)
		if($file_name =~ m/(.*)/s) {
			$file_name = $1;
		}

		if($mode =~ m/(.*)/s) {
			$mode = $1;
		}
		# What do we do about existing files?
		# 0, not processed so overwrites existing (is sent by NJS)
		# 1 => Do not overwrite existing
		# 2 => File must exist (not used by NJS Sept 03)
		# 3 => Append (new Sept 03)
        if(-e $file_name) {
			if($action == 1) {
				failed_report("File already exists: $file_name\n");
				return;
			}
        }
		else {
			if($action == 2) {
				failed_report("File does not exist: $file_name\n");
				return;
			}
		}

		# Interpret overwrite as -f so ..
		my $mmode = 6*64;    # that is 600 in octal
        chmod $mmode, $file_name; 

        # Open the file
		if($action == 3) {
			if( !open(OUTF, ">>$file_name") ) {
				failed_report("Putfiles cannot open the file for appending: $file_name.");
				return;
			}
		}
		else {
			if( !open(OUTF, ">$file_name") ) {
				failed_report("Putfiles cannot open the file for writing: $file_name.");
				return;
			}
		}

        print main::CMD_SOCK "TSI_OK\nENDOFMESSAGE\n";       # Put NJS into send mode for next command

        # Read packet length
        $packet_length = <main::CMD_SOCK>;
        chomp $packet_length;         # gets rid of ENFOFMESSAGE
        $packet_length =~ s/\n.*//g;  # keep only first line and without new line char

#        debug_report("Expecting $packet_length bytes for <$file_name>"); # here so that NJS gets to read it

        print main::CMD_SOCK "TSI_OK\nENDOFMESSAGE\n";       # Put NJS into send mode for next command

        # Read data
        while($packet_length > -1) {

            # No EOF in DATA stream, so have to count bytes
            $left = $packet_length;
            while($left >= $main::BUFFER_SIZE) {

                $len = read main::DATA_SOCK, $bytes, $main::BUFFER_SIZE;  # READ FROM THE DATA CHANNEL
                $left -= $len;

                $offset = 0;  # See Programming Perl p231
                while($len) { # Handle partial writes
                    $written = syswrite OUTF, $bytes, $len, $offset;
                    report_and_die("Putfiles has fatal write error: $!") unless defined $written;
                    $len -= $written;
                    $offset += $written;
                }
            }
            if($left > 0) {    

                $len = read main::DATA_SOCK, $bytes, $left;  # READ FROM THE DATA CHANNEL

                $offset = 0;  # See Programming Perl p231
                while($len) { # Handle partial writes
                    $written = syswrite OUTF, $bytes, $len, $offset;
                    report_and_die("Putfiles has fatal write error 2: $!") unless defined $written;
                    $len -= $written;
                    $offset += $written;
                }
            }

            # Read next packet length
            $packet_length  = <main::CMD_SOCK>;
            chomp $packet_length;         # gets rid of ENFOFMESSAGE
            $packet_length =~ s/\n.*//g;  # keep only first line and without new line char

#            debug_report("Expecting $packet_length bytes for <$file_name>"); # here so that NJS gets to read it

            print main::CMD_SOCK "TSI_OK\nENDOFMESSAGE\n";       # Put NJS into send mode for next command


        }
            
        close(OUTF);

		# Change mode of file to NJSs request
		$mode = $mode*64;    # that is $mode00 in octal
        chmod $mode, $file_name; 

        # Read the next file name
        $file_name = <main::CMD_SOCK>;
        chomp $file_name;         # gets rid of ENFOFMESSAGE
        $file_name =~ s/\n.*//g;  # keep only first line and without new line char

		($file_name,$mode) = split " ",$file_name;

    }

    ok_report("Putfiles finished executing OK");
 }
#
#                   Copyright (c) Fujitsu Ltd 2000 - 2004
#
#                Use and distribution is subject a License.
# A copy was supplied with the distribution (see documentation or the jar file).
#
# This product includes software developed by Fujitsu Limited (http://www.fujitsu.com).
